@@ -8,8 +8,12 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
-import math
 import sys
+
+if sys.version_info[:2] < (3, 8):
+    raise RuntimeError("The Array API standard requires Python 3.8 or later")
+
+import math
 from numbers import Real
 from types import SimpleNamespace
 from typing import (
@@ -65,10 +69,10 @@ __all__ = [
 ]
 
 
-RELEASED_VERSIONS = ("2021.12",)
+RELEASED_VERSIONS = ("2021.12", "2022.12")
 NOMINAL_VERSIONS = RELEASED_VERSIONS + ("draft",)
 assert sorted(NOMINAL_VERSIONS) == list(NOMINAL_VERSIONS)  # sanity check
-NominalVersion = Literal["2021.12", "draft"]
+NominalVersion = Literal["2021.12", "2022.12", "draft"]
 assert get_args(NominalVersion) == NOMINAL_VERSIONS  # sanity check
 
 
@@ -190,7 +194,7 @@ def _from_dtype(
     allow_subnormal: Optional[bool] = None,
     exclude_min: Optional[bool] = None,
     exclude_max: Optional[bool] = None,
-) -> st.SearchStrategy[Union[bool, int, float]]:
+) -> st.SearchStrategy[Union[bool, int, float, complex]]:
     """Return a strategy for any value of the given dtype.
 
     Values generated are of the Python scalar which is
@@ -293,31 +297,22 @@ def _from_dtype(
 
         return st.floats(width=finfo.bits, **kw)
     else:
-        # A less-inelegant solution to support complex dtypes exists, but as
-        # this is currently a draft feature, we might as well wait for
-        # discussion of complex inspection to resolve first - a better method
-        # might become available soon enough.
-        # See https://github.com/data-apis/array-api/issues/433
-        for attr in ["float32", "float64", "complex64"]:
-            if not hasattr(xp, attr):
-                raise NotImplementedError(
-                    f"Array module {xp.__name__} has no dtype {attr}, which is "
-                    "currently required for xps.from_dtype() to work with "
-                    "any complex dtype."
-                )
-        component_dtype = xp.float32 if dtype == xp.complex64 else xp.float64
-
-        floats = _from_dtype(
-            xp,
-            api_version,
-            component_dtype,
+        finfo = xp.finfo(dtype)
+        # See above comment on FTZ inference. We explicitly infer with a
+        # complex array, in case complex arrays have different FTZ behaviour
+        # than arrays of the respective composite float.
+        if allow_subnormal is None:
+            subnormal = next_down(finfo.smallest_normal, width=finfo.bits)
+            x = xp.asarray(complex(subnormal, subnormal), dtype=dtype)
+            builtin_x = complex(x)
+            allow_subnormal = builtin_x.real != 0 and builtin_x.imag != 0
+        return st.complex_numbers(
             allow_nan=allow_nan,
             allow_infinity=allow_infinity,
             allow_subnormal=allow_subnormal,
+            width=finfo.bits * 2,
         )
 
-        return st.builds(complex, floats, floats)  # type: ignore[arg-type]
-
 
 class ArrayStrategy(st.SearchStrategy):
     def __init__(
@@ -902,7 +897,7 @@ def make_strategies_namespace(
         check_argument(
             isinstance(xp.__array_api_version__, str)
             and xp.__array_api_version__ in RELEASED_VERSIONS,
-            f"{xp.__array_api_version__=}, but xp.__array_api_version__ must "
+            f"xp.__array_api_version__={xp.__array_api_version__!r}, but it must "
             f"be a valid version string {RELEASED_VERSIONS}. {not_available_msg}",
         )
         api_version = xp.__array_api_version__
@@ -910,8 +905,8 @@ def make_strategies_namespace(
     else:
         check_argument(
             isinstance(api_version, str) and api_version in NOMINAL_VERSIONS,
-            f"{api_version=}, but api_version must be None, or a valid version "
-            f"string {RELEASED_VERSIONS}. {not_available_msg}",
+            f"api_version={api_version!r}, but it must be None, or a valid version "
+            f"string in {RELEASED_VERSIONS}. {not_available_msg}",
         )
         inferred_version = False
     try:
@@ -941,7 +936,7 @@ def make_strategies_namespace(
         allow_subnormal: Optional[bool] = None,
         exclude_min: Optional[bool] = None,
         exclude_max: Optional[bool] = None,
-    ) -> st.SearchStrategy[Union[bool, int, float]]:
+    ) -> st.SearchStrategy[Union[bool, int, float, complex]]:
         return _from_dtype(
             xp,
             api_version,  # type: ignore[arg-type]
@@ -955,6 +950,28 @@ def make_strategies_namespace(
             exclude_max=exclude_max,
         )
 
+    # torch.full() does not accept integers as the shape argument (n.b.
+    # technically "size" in torch), but such behaviour is expected in
+    # xps.arrays(), so we copy xp and patch in a working function.
+    if xp is sys.modules.get("torch", object()):
+
+        class PatchedArrayModule:
+            def __getattr__(self, attr):
+                return getattr(xp, attr)
+
+            @property
+            def __name__(self):
+                return "torch<modified>"
+
+            def full(self, shape, *a, **kw):
+                if isinstance(shape, int):
+                    shape = (shape,)
+                return xp.full(shape, *a, **kw)
+
+        arrays_xp = PatchedArrayModule()
+    else:
+        arrays_xp = xp
+
     @defines_strategy(force_reusable_values=True)
     def arrays(
         dtype: Union[
@@ -967,7 +984,7 @@ def make_strategies_namespace(
         unique: bool = False,
     ) -> st.SearchStrategy:
         return _arrays(
-            xp,
+            arrays_xp,
             api_version,  # type: ignore[arg-type]
             dtype,
             shape,
@@ -1124,7 +1141,7 @@ if np is not None:
 
     mock_xp = SimpleNamespace(
         __name__="mock",
-        __array_api_version__="2021.12",
+        __array_api_version__="2022.12",
         # Data types
         int8=np.int8,
         int16=np.int16,
